@@ -39,8 +39,6 @@
 #include <sys/time.h>
 #include <sys/stat.h>
 
-#define APP_NAME		"smemstat"
-
 #define UNAME_HASH_TABLE_SIZE	(521)
 #define PROC_HASH_TABLE_SIZE 	(503)
 
@@ -87,25 +85,91 @@ typedef struct __attribute__ ((__packed__)) mem_info_t {
 	int64_t		d_swap;		/* Delta swap */
 	bool		alive;		/* true if proc is alive */
 
-	struct mem_info_t *d_next;	/* sotted deltas by total */
+	struct mem_info_t *d_next;	/* sorted deltas by total */
 	struct mem_info_t *s_next;	/* sorted by total */
 	struct mem_info_t *next;	/* for free list */
 } mem_info_t;
 
 typedef struct pid_list {
-	pid_t		pid;
-	char 		*name;
-	struct pid_list	*next;
+	pid_t		pid;		/* process id */
+	char 		*name;		/* process name */
+	struct pid_list	*next;		/* next in list */
 } pid_list_t;
 
 static uname_cache_t *uname_cache[UNAME_HASH_TABLE_SIZE];
 static proc_info_t *proc_cache_hash[PROC_HASH_TABLE_SIZE];
+static const char *app_name = "smemstat";
 
 static bool stop_smemstat = false;	/* set by sighandler */
 static unsigned int opt_flags;		/* options */
 static mem_info_t *mem_info_cache = NULL;
 static pid_list_t *pids = NULL;
 
+/*
+ *  Attempt to catch a range of signals so
+ *  we can clean
+ */
+static const int signals[] = {
+	/* POSIX.1-1990 */
+#ifdef SIGHUP
+	SIGHUP,
+#endif
+#ifdef SIGINT
+	SIGINT,
+#endif
+#ifdef SIGQUIT
+	SIGQUIT,
+#endif
+#ifdef SIGILL
+	SIGILL,
+#endif
+#ifdef SIGABRT
+	SIGABRT,
+#endif
+#ifdef SIGFPE
+	SIGFPE,
+#endif
+#ifdef SIGSEGV
+	SIGSEGV,
+#endif
+#ifdef SIGTERM
+	SIGTERM,
+#endif
+#ifdef SIGUSR1
+	SIGUSR1,
+#endif
+#ifdef SIGUSR2
+	SIGUSR2,
+	/* POSIX.1-2001 */
+#endif
+#ifdef SIGBUS
+	SIGBUS,
+#endif
+#ifdef SIGXCPU
+	SIGXCPU,
+#endif
+#ifdef SIGXFSZ
+	SIGXFSZ,
+#endif
+	/* Linux various */
+#ifdef SIGIOT
+	SIGIOT,
+#endif
+#ifdef SIGSTKFLT
+	SIGSTKFLT,
+#endif
+#ifdef SIGPWR
+	SIGPWR,
+#endif
+#ifdef SIGINFO
+	SIGINFO,
+#endif
+#ifdef SIGVTALRM
+	SIGVTALRM,
+#endif
+	-1,
+};
+
 /*
  *  out_of_memory()
  *      report out of memory condition
@@ -206,10 +270,10 @@ static char *get_pid_comm(const pid_t pid)
 		return NULL;
 
 	if ((ret = read(fd, buffer, sizeof(buffer))) <= 0) {
-		close(fd);
+		(void)close(fd);
 		return NULL;
 	}
-	close(fd);
+	(void)close(fd);
 	buffer[ret-1] = '\0';
 
 	return strdup(buffer);
@@ -232,10 +296,10 @@ static char *get_pid_cmdline(const pid_t pid)
 		return NULL;
 
 	if ((ret = read(fd, buffer, sizeof(buffer))) <= 0) {
-		close(fd);
+		(void)close(fd);
 		return NULL;
 	}
-	close(fd);
+	(void)close(fd);
 
 	if (ret >= (ssize_t)sizeof(buffer))
 		ret = sizeof(buffer) - 1;
@@ -395,12 +459,12 @@ static struct timeval timeval_sub(const struct timeval *a, const struct timeval
 	_b.tv_usec = b->tv_usec;
 
 	if (a->tv_usec < _b.tv_usec) {
-		int nsec = ((_b.tv_usec - a->tv_usec) / 1000000) + 1;
+		suseconds_t nsec = ((_b.tv_usec - a->tv_usec) / 1000000) + 1;
 		_b.tv_sec += nsec;
 		_b.tv_usec -= (1000000 * nsec);
 	}
 	if (a->tv_usec - _b.tv_usec > 1000000) {
-		int nsec = (a->tv_usec - _b.tv_usec) / 1000000;
+		suseconds_t nsec = (a->tv_usec - _b.tv_usec) / 1000000;
 		_b.tv_sec -= nsec;
 		_b.tv_usec += (1000000 * nsec);
 	}
@@ -698,10 +762,10 @@ static int mem_get_by_proc(const pid_t pid, mem_info_t **mem)
 
 	/* Can't read it, no access rights? */
 	if (errno == EACCES) {
-		fclose(fp);
+		(void)fclose(fp);
 		return 0;
 	}
-	fclose(fp);
+	(void)fclose(fp);
 
 	if ((new_m = mem_cache_alloc()) == NULL)
 		return -1;
@@ -723,14 +787,14 @@ static int mem_get_by_proc(const pid_t pid, mem_info_t **mem)
 			if (sscanf(buffer + 5, "%9i", &new_m->uid) == 1) {
 				new_m->uname = uname_cache_find(new_m->uid);
 				if (new_m->uname == NULL) {
-					fclose(fp);
+					(void)fclose(fp);
 					return -1;
 				}
 				break;
 			}
 		}
 	}
-	fclose(fp);
+	(void)fclose(fp);
 
 	return 0;
 }
@@ -758,13 +822,13 @@ static int mem_get_all_pids(mem_info_t **mem, size_t *npids)
 		pid = (pid_t)strtoul(entry->d_name, NULL, 10);
 
 		if (mem_get_by_proc(pid, mem) < 0) {
-			closedir(dir);
+			(void)closedir(dir);
 			return -1;
 		}
 		(*npids)++;
 	}
 
-	closedir(dir);
+	(void)closedir(dir);
 
 	return 0;
 }
@@ -1032,10 +1096,10 @@ static int mem_dump_diff(
 }
 
 /*
- *  handle_sigint()
- *      catch SIGINT and flag a stop
+ *  handle_sig()
+ *      catch signals and flag a stop
  */
-static void handle_sigint(int dummy)
+static void handle_sig(int dummy)
 {
 	(void)dummy;    /* Stop unused parameter warning with -Wextra */
 
@@ -1110,7 +1174,6 @@ nomem:
 	out_of_memory("allocating pid list.\n");
 	pid_list_cleanup();
 	return -1;
-	
 }
 
 /*
@@ -1133,7 +1196,7 @@ static void show_usage(void)
 		"  -p proclist\tspecify comma separated list of processes to monitor\n"
 		"  -q\t\trun quietly, useful for -o output only\n"
 		"  -s\t\tshow short command information\n",
-		APP_NAME, VERSION, APP_NAME);
+		app_name, VERSION, app_name);
 }
 
 int main(int argc, char **argv)
@@ -1236,7 +1299,7 @@ int main(int argc, char **argv)
 			fprintf(stderr, "Cannot open json output file '%s'.\n", json_filename);
 			exit(EXIT_FAILURE);
 		}
-		fprintf(json_file, "{\n  \"" APP_NAME "\":{\n");
+		fprintf(json_file, "{\n  \"%s\":{\n", app_name);
 	}
 
 	if (count == 0) {
@@ -1246,6 +1309,9 @@ int main(int argc, char **argv)
 		mem_report_size();
 		goto tidy;
 	} else {
+		struct sigaction new_action;
+		int i;
+
 		/*
 		 *  Pre-cache, this way we reduce
 		 *  the amount of mem infos we alloc during
@@ -1259,10 +1325,26 @@ int main(int argc, char **argv)
 		duration.tv_usec = (suseconds_t)(duration_secs * 1000000.0) - (duration.tv_sec * 1000000);
 		whence.tv_sec = 0;
 		whence.tv_usec = 0;
-		gettimeofday(&tv1, NULL);
+		if (gettimeofday(&tv1, NULL) < 0) {
+			fprintf(stderr, "gettimeofday failed: errno=%d (%s)\n",
+				errno, strerror(errno));
+			exit(EXIT_FAILURE);
+		}
 
 		printf("Change in memory (average per second):\n");
-		signal(SIGINT, &handle_sigint);
+
+		memset(&new_action, 0, sizeof(new_action));
+		for (i = 0; signals[i] != -1; i++) {
+			new_action.sa_handler = handle_sig;
+			sigemptyset(&new_action.sa_mask);
+			new_action.sa_flags = 0;
+
+			if (sigaction(signals[i], &new_action, NULL) < 0) {
+				fprintf(stderr, "sigaction failed: errno=%d (%s)\n",
+					errno, strerror(errno));
+				exit(EXIT_FAILURE);
+			}
+		}
 
 		if (json_file) {
 			fprintf(json_file, "    \"periodic-samples\":[\n");
@@ -1272,7 +1354,11 @@ int main(int argc, char **argv)
 			struct timeval tv;
 			int ret;
 
-			gettimeofday(&tv2, NULL);
+			if (gettimeofday(&tv2, NULL) < 0) {
+				fprintf(stderr, "gettimeofday failed: errno=%d (%s)\n",
+					errno, strerror(errno));
+				exit(EXIT_FAILURE);
+			}
 
 			tv = timeval_add(&duration, &whence);
 			tv = timeval_add(&tv, &tv1);
@@ -1324,7 +1410,7 @@ tidy:
 
 	if (json_file) {
 		fprintf(json_file, "  }\n}\n");
-		fclose(json_file);
+		(void)fclose(json_file);
 	}
 
 	exit(EXIT_SUCCESS);
